angular.module('pnpAuthentication')
.service('identityService', ['loginRA', '$state', 'notificationService', 'localStorageService', '$ionicLoading', '$rootScope',
	function(loginRA, $state, notificationService, localStorageService, $ionicLoading, $rootScope) {	
	this.user = null;
	this.redirectUrl = "";
	this.isLoggedIn = 'false';
	var ids = this;
	this.getRedirectString = function() {
		return redirect;
	}

	this.pushRedirectString = function(redirectUrl) {
		this.redirectUrl = redirectUrl;
	}

	this.popRedirectString = function() {
		var temp = this.redirectUrl;
		this.redirectUrl = "";
		return temp;
	}

	this.getHeaderCartCount = function() {
		loginRA.one('my-save-lists','random').customGET('cartSummary').then(function(response){
					$rootScope.cart_count = response.total_items;
		});
		return $rootScope.cart_count;
	}

	this.login = function(loginForm) {
		return loginRA.one("user-sessions",'random').all('login').post(loginForm).then(function(response) {
			if (response.meta.result == 'success') {
				localStorageService.set('IbCustInfo', JSON.stringify(response.customer));
				localStorageService.set('AuthKey', response.auth_key);
				ids.getHeaderCartCount();
				notificationService.add("Login Successful");
			}
			else {
				if (response.meta['field_errors'] != undefined) {
					notificationService.add(response.meta['field_errors'][0]['message']);
				}
				else {
					notificationService.add(response.meta['message']);
				}
			}
			return response.meta.result;
		});
	}

	this.getUser = function() {
		if(!this.user) {
			this.user = localStorageService.get('IbCustInfo');
		}
		return this.user;
	}

	this.isLoggedIn = function() {
		if(!this.user) {
			this.user = this.getUser();
		}
		return this.user?true:false;
	}

	this.googlelogout = function() {
		if (window.plugins.googleplus) {
			window.plugins.googleplus.logout(
		    	function (msg) {console.log(msg);}
			);
		}
	}

	this.logout = function() {
		this.googlelogout();
		return loginRA.one('user-sessions','random').customGET('logout').then(function(response) {
			if (response.meta.result == 'success') {
				notificationService.add("Logout Successful");
				localStorageService.set('IbCustInfo', null);
				localStorageService.set('AuthKey', null);
				localStorageService.set('WishList', null);
				$rootScope.cart_count = 0;
			}
			else if (response.meta.result == 'error') {
				notificationService.add(response.meta['message']);
				localStorageService.set('IbCustInfo', null);
				localStorageService.set('AuthKey', null);
			}
			return response.meta.result;
		});
	}	

	this.register = function(formData) {
		formData["confirm_password"] = formData["password"];
		return loginRA.one('user-sessions','random').all('register').post(formData).then(function(response) {
			if (response.meta.result == 'success') {
				localStorageService.set('IbCustInfo', JSON.stringify(response.customer));
				localStorageService.set('AuthKey', response.auth_key);
				notificationService.add("Registration Successful");				
			}
			else {
				if (response.meta['field_errors'] != undefined && response.meta['field_errors'][0]['field'] == "email") {
					notificationService.add(response.meta['field_errors'][0]['message']);
				}
				else {
					notificationService.add(response.meta['message']);
				}				
			}
			return response.meta.result;
		});
	}
	
	this.forgotPwd = function(formData) {
		return loginRA.one('user-sessions','random').all('forgotPassword').post(formData).then(function(response) {
			notificationService.add(response.meta['message']);
			return response.meta.result;			
		});
	}

	this.resetPwd = function(formData) {
		return loginRA.one('user-sessions','random').all('resetPassword').post(formData).then(function(response) {
			if (response.meta.result == 'success') {
				localStorageService.set('IbCustInfo', JSON.stringify(response.customer));
				localStorageService.set('AuthKey', response.auth_key);
				notificationService.add(response.meta['message']);
			}
			else {
				notificationService.add(response.meta['message']);
			}			
			return response.meta.result;			
		});
	}

	this.loginByAuthKey = function() {
		var formData = {"auth_key" : localStorageService.get('AuthKey')}
		return loginRA.one('user-sessions','random').all('loginByAuthKey').post(formData).then(function(response) {
			if (response.meta.result == 'success') {
				localStorageService.set('IbCustInfo', JSON.stringify(response.customer));
				localStorageService.set('AuthKey', response.auth_key);
			} 
			return response.meta.result;
		});
	}

	this.loginByGoogle = function(obj) {
		var loginForm = {};
		loginForm.email = obj.email;
		loginForm.user_id = obj.userId;
		loginForm.auth_token = obj.oauthToken != "" && obj.oauthToken != null ? obj.oauthToken : obj.accessToken;	
		$ionicLoading.show({
      template: '<img src="img/ib-apploader.gif">'
    });
		return loginRA.one("user-sessions",'random').all('loginByGoogle').post(loginForm).then(function(response) {
			if (response.meta.result == 'success') {
				localStorageService.set('IbCustInfo', JSON.stringify(response.customer));
				localStorageService.set('AuthKey', response.auth_key);
				notificationService.add("Login Successful");
			}
			else {
				if (response.meta['field_errors'] != undefined) {
					notificationService.add(response.meta['field_errors'][0]['message']);
				}
				else {
					notificationService.add(response.meta['message']);
				}
			}
			$ionicLoading.hide();
			return response.meta.result;
		});
	}
		
	this.loginByFacebook = function (userId, oauthToken) {
		var loginForm = {};
		loginForm.user_id = userId;
		loginForm.auth_token = oauthToken;
		return loginRA.one("user-sessions",'random').all('loginByFacebook').post(loginForm).then(function(response) {
			if (response.meta.result == 'success') {
				localStorageService.set('IbCustInfo', JSON.stringify(response.customer));
				localStorageService.set('AuthKey', response.auth_key);
				notificationService.add("Login Successful");
			}
			else {
				if (response.meta['field_errors'] != undefined) {
					notificationService.add(response.meta['field_errors'][0]['message']);
				}
				else {
					notificationService.add(response.meta['message']);
				}
			}
			return response.meta.result;
		});
	}
}])
