angular.module('pnpAutocomplete', ['pnpCore'])
.directive('ionAutocomplete', [
    '$ionicTemplateLoader', '$ionicBackdrop', '$rootScope', '$document', '$q', '$parse', '$http', '$cacheFactory', 'babRA', '$location', '$ionicHistory', 'browseNodeService', '$stateParams', '$ionicPlatform',
    function ($ionicTemplateLoader, $ionicBackdrop, $rootScope, $document, $q, $parse, $http, $cacheFactory, babRA, $location, $ionicHistory, browseNodeService, $stateParams, $ionicPlatform) {
        return {
            require: '?ngModel',
            restrict: 'E',
            template: '<span class="button button-icon icon ion-ios-search-strong pull-right" ></span>',
            replace: true,
            scope: {
                itemsMethod: '&',
                itemsMethodValueKey: '@',
                itemValueKey: '@',
                itemViewValueKey: '@'
            },
            link: function (scope, element, attrs, ngModel) {
                // do nothing if the model is not set
                if (!ngModel) return;

                // set placeholder on element
                scope.placeholder = '';
                if (attrs.placeholder) {
                    scope.placeholder = attrs.placeholder;
                    element.attr('placeholder', attrs.placeholder);
                }

                // set cancel button label
                scope.cancelLabel = '<button class="ion-autocomplete-cancel button button-clear ion-arrow-left-c"></button>';
                if (attrs.cancelLabel) {
                  scope.cancelLabel = attrs.cancelLabel;
                }
                
                // the items of the list
                scope.items = [];
                scope.searchQuery = '';
                scope.c = new LRUCache(5);
                scope.cachetemp = localStorage.getItem('ibacHistory');
                scope.listIcon = 'img/ic_history.png';
                if(scope.cachetemp !== null) {
                  JSON.parse(scope.cachetemp).forEach(function(i,v,x){
                    scope.c.set(i.key,i.value);
                    scope.items.push(i.value);
                  });
                }
                scope.saveTOCache = function(key, value) {
                  scope.c.set(key, value);
                  localStorage.setItem('ibacHistory', scope.c.toJSON());
                };
                // returns the value of an item
                scope.getItemValue = function (item, key) {
                  var itemValue;
                  if (key && angular.isObject(item)) {
                    itemValue = $parse(key)(item);
                  } else {
                    itemValue = item;
                  }
                  return itemValue;
                };

                scope.clearSearch= function() {
                  scope.searchQuery = '';
                }
                scope.getItemValueHighlight = function (item, key) {
                  var itemValue;
                  function makeMatchedBold(match) {
                    return ['<b>', match.toLowerCase(), '</b>'].join('');
                  }
                  if (key && angular.isObject(item)) {
                    itemValue = $parse(key)(item);
                    if(scope.searchQuery !== "") {
                      var re = new RegExp(scope.searchQuery,"g");
                      itemValue = itemValue.replace(re, makeMatchedBold);
                    }
                  } else {
                    itemValue = item;
                  }
                  if(key == 'category' && itemValue != null) {
                    itemValue = "in " + itemValue
                  }
                  return itemValue;
                };
                scope.gethrefpath = function (item) {
                  return "browse_nodes"+item.url+"&top_categories=true";
                }
                
                scope.moveToSearchBar = function(item) {
                    scope.searchQuery = item;
                }
               
                // compile the popup template
                $ionicTemplateLoader.compile({
                    templateUrl: 'lib/pnp-autocomplete/templates/show.html',
                    scope: scope,
                    appendTo: $document[0].body
                }).then(function (compiledTemplate) {

                    // get the compiled search field
                    var searchInputElement = angular.element(compiledTemplate.element.find('input'));
                    // function which selects the item, hides the search container and the ionic backdrop
                     compiledTemplate.scope.homeSearch = function(searchQuery) {
                      if(searchQuery != "") {
                        var item = {"category": "Home", "value" : searchQuery, "url": "/Home/search?q="+searchQuery+"&predict=true"};
                        compiledTemplate.scope.selectItem(item);
                      }
                    }

                    compiledTemplate.scope.selectItem = function (item) {

                        // clear the items and the search query
                        // compiledTemplate.scope.items = [];
                        compiledTemplate.scope.searchQuery = item.value;

                        // set the view value and render it
                        // ngModel.$setViewValue(item);
                        // ngModel.$render();
                        compiledTemplate.scope.saveTOCache(item.value, item);
                        // hide the container and the ionic backdrop
                        compiledTemplate.element.css('display', 'none');
                        $ionicBackdrop.release();
                        $location.url(compiledTemplate.scope.gethrefpath(item));
                    };

                    // watcher on the search field model to update the list according to the input
                    compiledTemplate.scope.$watch('searchQuery', function (query) {

                        // if the search query is empty, clear the items
                        if(query == '') {
                            compiledTemplate.scope.items = compiledTemplate.scope.c.toCacheItems();
                            compiledTemplate.scope.listIcon = 'img/ic_history.png'
                        }

                        if (query && angular.isFunction(compiledTemplate.scope.itemsMethod)) {

                            // // convert the given function to a $q promise to support promises too
                            // var promise = $q.when(compiledTemplate.scope.itemsMethod({query: query}));

                            // promise.then(function (promiseData) {
                            //     // set the items which are returned by the items method
                            //     compiledTemplate.scope.items = compiledTemplate.scope.getItemValue(promiseData,
                            //         compiledTemplate.scope.itemsMethodValueKey);
                            // }, function (error) {
                            //     // reject the error because we do not handle the error here
                            //     return $q.reject(error);
                            // });
                            compiledTemplate.scope.listIcon = 'img/ic_search.png'
                            $http.jsonp("http://ac.infibeam.com/ac.json?term="+query+"&callback=JSON_CALLBACK", {cache: true, ignoreLoadingBar: true})
                              .then(function(result) {
                                compiledTemplate.scope.items = compiledTemplate.scope.getItemValue({items: result.data}, compiledTemplate.scope.itemsMethodValueKey);
                            });
                            compiledTemplate.scope.staticItem = {"category": "Home", "value" : query, "url": "/Home/search?q="+query+"&predict=true"};
                        }
                    });

                    // click handler on the input field which
                    var onClick = function (event) {
                        $ionicHistory.nextViewOptions({
                          disableAnimate: true
                        });
                        // prevent the default event and the propagation
                        event.preventDefault();
                        event.stopPropagation();

                        // show the ionic backdrop and the search container
                        $ionicBackdrop.retain();
                        compiledTemplate.element.css('display', 'block');

                        // focus on the search input field
                        searchInputElement[0].focus();
                        setTimeout(function () {
                            searchInputElement[0].focus();
                        }, 0);
                    };

                    // bind the onClick handler to the click and touchend events
                    element.bind('click', onClick);
                    element.bind('touchend', onClick);

                    // cancel handler for the cancel button which clears the search input field model and hides the
                    // search container and the ionic backdrop
                    compiledTemplate.element.find('button').bind('click', function (event) {
                        // compiledTemplate.scope.searchQuery = '';
                        $ionicBackdrop.release();
                        compiledTemplate.element.css('display', 'none');
                    });
                });

                // render the view value of the model
                ngModel.$render = function () {
                    var elementValue = scope.getItemValue(ngModel.$viewValue, scope.itemViewValueKey);
                    element.val(elementValue)
                };

                // set the view value of the model
                ngModel.$formatters.push(function (modelValue) {
                    return scope.getItemValue(modelValue, scope.itemViewValueKey);
                });

                // set the model value of the model
                ngModel.$parsers.push(function (viewValue) {
                    return scope.getItemValue(viewValue, scope.itemValueKey)
                });

            }
        };
    }
])
.directive('ngEnter', function() {
        return function(scope, element, attrs) {
            element.bind("keydown keypress", function(event) {
                if(event.which === 13) {
                        scope.$apply(function(){
                                scope.$eval(attrs.ngEnter);
                        });
                        
                        event.preventDefault();
                }
            });
        };
})
;