angular.module('pnpContent')

//base widget directive - dont modify unless you want changes across widgets
.directive('ibWidget', ['$compile', function($compile){
  
  function camelToDash(str) {
    return str.replace(/\W+/g, '-').replace(/([a-z\d])([A-Z])/g, '$1-$2').toLowerCase();
  };
	var ibWidget = {};
	ibWidget.restrict = 'E';
	ibWidget.scope = { content : '=' };
	ibWidget.compile = function(element, attributes) {
		var linkFunction = function(scope, element, attributes) {
			if(!scope.content.ng_class_name) {
				scope.content.ng_class_name = camelToDash('ib' + scope.content.type);
			}
			if(!scope.content.viewType) {
				scope.content.viewType = scope.content.data[':view_type']||'default';
			}
			if(scope.content && scope.content.ng_class_name) {
				var tag = '<div class="widget '+ scope.content.ng_class_name + ' ' + (scope.content.data[':custom_class_name']||'') +'" view-type="'+ scope.content.viewType +'" content="::content"></div>';
				element.replaceWith($compile(tag)(scope));
			}
    }
    return linkFunction;
	};
	return ibWidget;
}])

//html widget directive
.directive('ibHtmlWidget', ['$compile', function($compile){
	var htmlWidget = {};
	htmlWidget.restrict = 'EC';
	htmlWidget.scope = { content : '=' };
	htmlWidget.compile = function(element, attributes){
		var linkFunction = function(scope, element, attributes) {
			if(scope.content) {
				element.addClass('item');
				element.append($compile( scope.content.data[':html'] )(scope));
			}
    }
    return linkFunction;
	};
	return htmlWidget;
}])

//banner widget directive
.directive('ibBannerWidget', ['$compile', '$window', 'retinaImageGet', function($compile, $window, retinaImageGet){
	var bannerWidget = {};
	bannerWidget.restrict = 'EC';
	bannerWidget.scope = { content : '=' };
	bannerWidget.compile = function(element, attributes){
		var linkFunction = function(scope, element, attributes) {
			if(scope.content) {
				var tag = "<a href='#" + scope.content.data[':link'] + "' afkl-lazy-image='" + 
									retinaImageGet(scope.content.data[':image_path'], $window.innerWidth, '', 40) +
									"' class='afkl-lazy-wrapper' style='" + scope.content.data[':custom_css'] + "'></a>";
				element.append($compile(tag)(scope));
			}
    }
    return linkFunction;
	};
	// bannerWidget.template = "<div afkl-lazy-image='{{data[':image_path']}}'" + 
	// 												"afkl-lazy-image-options='{'alt': 'caption for image'}'" +
	// 												"style='max-width:100%;height:auto;{{data[':custom_css']}}'>" +
	// 												"</div>" ;
	return bannerWidget;
}])

//product list widget directive
.directive('ibProductListWidget', ['$compile', function($compile){
	var productListWidget = {};
	productListWidget.restrict = 'EC';
	productListWidget.scope = { content : '=' };
	productListWidget.controller = function($scope, $ionicScrollDelegate, $timeout){

		//this name will be used as delegate handle for ion scroll and other id purpose in views
		$scope.name = 'plw:' + $scope.content['$$hashKey'];
		//remove out of stock products
		$scope.content.variants = _.filter($scope.content.variants, function(variant) { return variant.item_availability !== "OutOfStock" });

	};
	productListWidget.templateUrl =  function(tElem, tAttr) {
		var typeHash = {
			'slidetype' : 'lib/pnp-content/templates/product_list_widget_slide_view.html',
			'gridtype' : 'lib/pnp-content/templates/product_list_widget_grid_view.html',
			'listtype' : 'lib/pnp-content/templates/product_list_widget_list_view.html',
			'cardtype' : 'lib/pnp-content/templates/product_list_widget_card_view.html'
		};
		if(tAttr.viewType && typeHash[tAttr.viewType])
			return typeHash[tAttr.viewType];
		else
			return typeHash['slidetype'];
	};
	return productListWidget;
}])

//carousel widget directive
.directive('ibCarouselWidget', ['$compile', function($compile){
	var carouselWidget = {};
	carouselWidget.restrict = 'EC';
	carouselWidget.scope = { content : '=' };
	carouselWidget.templateUrl = 'lib/pnp-content/templates/carousel_widget.html';
	carouselWidget.controller = function($scope, $window, retinaImageGet) {
		$scope.retinaImageGet = retinaImageGet;
		
		//initialize aspect ratio
		var aspect_ratio = 1.97530864;
		if($scope.content.data.widgets 
		&& $scope.content.data.widgets[0] 
		&& $scope.content.data.widgets[0][':aspect_ratio']){
			aspect_ratio = $scope.content.data.widgets[0][':aspect_ratio'];
		}
		$scope.aspect_ratio = ( typeof(aspect_ratio) == "number" ) ? aspect_ratio : aspect_ratio[0];

		$scope.imageOptWidth = $window.innerWidth;
		$scope.carouselStyles = { 'height' : (Math.ceil($scope.imageOptWidth / $scope.aspect_ratio))+'px' };

	}
	return carouselWidget;
}])

//banner list widget directive
.directive('ibBannerListWidget', ['$compile', function($compile){
	var banner_list_widget = {};
	banner_list_widget.restrict = 'EC';
	banner_list_widget.scope = { content : '=' };
	banner_list_widget.controller = function($scope, $ionicScrollDelegate, $window, retinaImageGet){
		//this name will be used as delegate handle for ion scroll and other id purpose in views
		$scope.name = 'blw:' + $scope.content['$$hashKey'];
		$scope.retinaImageGet = retinaImageGet;
		
		//initialize aspect ratio
		var aspect_ratio = 0.9;
		if($scope.content.data.widgets 
		&& $scope.content.data.widgets[0] 
		&& $scope.content.data.widgets[0][':aspect_ratio']){
			aspect_ratio = $scope.content.data.widgets[0][':aspect_ratio'];
		}
		$scope.aspect_ratio = ( typeof(aspect_ratio) == "number" ) ? aspect_ratio : aspect_ratio[0];

		// set image width
		var splitPercertageHash = { 'slidetype' : 66, 'gridtype' : 50, 'cardtype' : 100 };
		var paddingHash = { 'slidetype' : 34, 'gridtype' : 12, 'cardtype' : 34 };
		var splitPercertage = $scope.content.viewType && splitPercertageHash[$scope.content.viewType]
			? splitPercertageHash[$scope.content.viewType] : splitPercertageHash['gridtype'];
		var padding = $scope.content.viewType && paddingHash[$scope.content.viewType]
			? paddingHash[$scope.content.viewType] : paddingHash['gridtype'];
		$scope.imageOptWidth = Math.floor($window.innerWidth * (splitPercertage/100) - padding)

		//style
		var ionScrollHeight = ( Math.ceil( $scope.imageOptWidth / $scope.aspect_ratio + padding ) );
		ionScrollHeight += (20-(ionScrollHeight%20));
		$scope.carouselStyles = { 'height' : ionScrollHeight+'px' };
	};
	banner_list_widget.templateUrl = function(tElem, tAttr){
		var typeHash = {
			'gridtype' : 'lib/pnp-content/templates/banner_list_widget_grid_view.html',
			'slidetype' : 'lib/pnp-content/templates/banner_list_widget_slide_view.html',
			'cardtype' : 'lib/pnp-content/templates/banner_list_widget_card_view.html'
		};
		if(tAttr.viewType && typeHash[tAttr.viewType])
			return typeHash[tAttr.viewType];
		else
			return typeHash['gridtype'];
	};
	return banner_list_widget;
}])

//navigation widget directive
.directive('ibNavigationWidget', ['$compile', function($compile){
	var navigation_widget = {};
	navigation_widget.restrict = 'EC';
	navigation_widget.scope = { content : '=' };
	navigation_widget.controller = function($scope) {
		$scope.showMore = false ;
		$scope.toggleMore = function(){
			$scope.showMore = !$scope.showMore;
		}
	}
	navigation_widget.templateUrl = function(tElem, tAttr) {
		var typeHash = {
			'menutype' : 'lib/pnp-content/templates/navigation_widget_menu_view.html',
			'tagtype' : 'lib/pnp-content/templates/navigation_widget_tag_view.html',
			'listtype' : 'lib/pnp-content/templates/navigation_widget_list_view.html'
		};
		if(tAttr.viewType && typeHash[tAttr.viewType])
			return typeHash[tAttr.viewType];
		else
			return typeHash['tagtype'];
	};
	return navigation_widget;
}])

.directive('ibNavWidgetSplitView', [function(){
	var nav_widget_split = {};
	nav_widget_split.restrict = 'EC';
	nav_widget_split.templateUrl = 'lib/pnp-content/templates/navigation_widget_split_view.html';
	nav_widget_split.scope = { data : '=' };
	nav_widget_split.controller = function($scope) {
		$scope.navdata = $scope.data.split('||||');
	}
	return nav_widget_split;
}])
