pnpCore
//abstract product factory wrapper to add common methods accross product classes
.factory('productDecorator', ['productReviewsService', function(productReviewsService){
  return function(product){

    if(!product) return { 'error' : "Something went wrong! Please try again later." };
    
    product.reviews = null;
    product.reviewsService = productReviewsService;

    product.set_current_image = function(image_id) {
      product.current_image = _.find(product.catalog_images, {'image_id' : image_id});  
    }

    product.set_current_variant = function(variant_id) {
      if(product.variants_map && product.variants_map[variant_id])
        product.current_variant = product.variants_map[variant_id];
      else
        product.current_variant = _.find(product.variants, {'sku' : variant_id});

      if(product.current_variant.default_image_id)
        product.set_current_image(product.current_variant.default_image_id);
      else if(product.catalog_images)
        product.set_current_image( product.catalog_images[0]['image_id'] );
      
      _(product.current_variant.catalog_options).forEach(function(catalog_option, index){
        product.co_ordering[index].selected_option = _.find(product.co_ordering[index].options, function(option){
          return option.value === catalog_option.value; //search condition
        });
      });
    }

    product.getOverallRatingStars = function(){
      var roundedRating = Math.round(product.rating_summary.total / product.rating_summary.number);
      roundedRating-=(roundedRating%5);
      var stars = [];
      var full = (roundedRating - (roundedRating%10))/10;
      var half = (roundedRating%10)/5;
      for (var i = 0; i < 5; i++) {
        if(i < full)  
          stars.push('full');
        else if(i == full && half)
          stars.push('half');
        else
          stars.push('empty');
      };
      return stars;
    }
    product.getOverallRating = function(){
      var roundedRating = product.rating_summary.total / product.rating_summary.number;
      var roundedRating = Math.round(roundedRating)/10;
      return roundedRating;
    }

    //will be used by init - to initializing variant maps for catalog options
    product.init_mapping = function() {
      product.variants_map = {};
      product.catalog_options_tree = {};
      product.co_ordering = [];
      var avail_options = {};
      _(product.variants).forEach(function(variant){
        var map = [];
        var opt_tree = product.catalog_options_tree;
        _(variant.catalog_options).forEach(function(catalog_option, index){
          //options map creation
          map.push(catalog_option.value);
          //catalog ordering creation and init
          if(!product.co_ordering[index]) product.co_ordering[index] = {};
          if(!product.co_ordering[index].type) product.co_ordering[index].type = catalog_option.type;
          if(!product.co_ordering[index].options) product.co_ordering[index].options = [];
          if(!avail_options[catalog_option.value]) {
            avail_options[catalog_option.value] = true;
            product.co_ordering[index].options.push(catalog_option);
          }
          //tree creation and init
          if(!opt_tree.hasOwnProperty(catalog_option.value)){
            opt_tree[catalog_option.value] = (index==variant.catalog_options.length-1) ? variant.sku : {};
          }
          opt_tree = opt_tree[catalog_option.value];
        });
        product.variants_map[variant.sku] = variant; //map to variant init
      });
    }

    product.cleanups = function(){
      //convert recommended products to product list widget
      if(product.recommended_products && product.recommended_products.length){
        var plw = {};
        plw.name = 'Products You May Be Interested In';
        plw.type = "ProductListWidget";
        plw.data = { ':view_type' : "slidetype", ':title': plw.name };
        plw.variants = product.recommended_products;
        product.widgets = [plw];
      }
      //variant level -->
      _(product.variants).forEach(function(variant){
        if(variant.offer_title){ //offer html trims
          variant.offer_title = variant.offer_title.replace(/\.?(<[Bb][Rr]\s*\/?>\s*)+/g, '<br/>').trim();
        }
        variant.discount_percent = Math.ceil((variant.list_price.value - variant.wsp.value) / variant.list_price.value * 100);
      });
    }

    //will be used by init and changedOption to reset catalog options as per current variant
    product.refresh_co_ordering = function(selected_options) {
      var opt_tree = product.catalog_options_tree;
      if( selected_options.length == 1 ) { // do when catalog options with 1 level
        _( product.co_ordering[0].options ).forEach(function(option){
          option.available = true;
        });
        return;
      }
      // do when catalog options exceeds 1 level
      _( selected_options.slice(0,selected_options.length-1) ).forEach(function(selected, index){
        if(index==0) //all root options always available
        {
          _( product.co_ordering[index].options ).forEach(function(option){
            option.available = true;
          });
        }
        var avail_options = Object.keys(opt_tree[selected]);
        _( product.co_ordering[index+1].options ).forEach(function(option){
          option.available = _.find(avail_options, function(ao){ return ao === option.value }) ? true : false;
        });
        opt_tree = opt_tree[selected];
      });
    }

    //Methods for Catalog Options Begin
    product.setPreVisit = function(selected_options) {
      _(selected_options).forEach(function(v,i){
        if(i <= product.changedOptionIndex) {
          product.pre_visits[i][v] = selected_options;
        }
      });
    }

    product.getPreVisit = function(selected_options) {
      var current_selection = selected_options[product.changedOptionIndex];
      if(product.pre_visits[product.changedOptionIndex][current_selection]){
        var parent_opts_flag = true;
        var probable_previsit = product.pre_visits[product.changedOptionIndex][current_selection];
        _(selected_options).forEachRight(function(v,i){
          if(i < product.changedOptionIndex){
            parent_opts_flag = (selected_options[i] == probable_previsit[i] ? parent_opts_flag : false );
          }
        });
        if(parent_opts_flag) return probable_previsit;
      }
      return undefined;
    }

    product.fallbackOption = function(selected_options) {
      //try getting fallback from prev visits
      var fallback_options = product.getPreVisit(selected_options);
      if(fallback_options) return fallback_options;
      //fallback option if variant for selected option is not available
      fallback_options = [];
      var opt_tree = product.catalog_options_tree;
      _(selected_options).forEach(function(selected, index){
        if(opt_tree.hasOwnProperty(selected)){
          fallback_options.push(selected);
          opt_tree = opt_tree[selected];
        } else {
          var fallback = undefined;
          if(selected_options[index+1] && product.changedOptionIndex == index) //check if next option order is available in one of the current
            fallback = _.find(Object.keys(opt_tree), function(k){ return opt_tree[k].hasOwnProperty(selected_options[index+1]) });
          if(!fallback) fallback = _.first(Object.keys(opt_tree)); //select first key otherwise
          fallback_options.push(fallback);
          opt_tree = opt_tree[fallback];
        }
      });
      return fallback_options;
    }

    product.searchOption = function(selected_options) {
      var catalog_option_path = product.catalog_options_tree;
      _(selected_options).forEach(function(opt) {
        catalog_option_path = catalog_option_path[opt] ? catalog_option_path[opt] : false;
        if(!catalog_option_path) return false;
      });
      return catalog_option_path; // return variant sku if found in path else false
    }

    product.changedOption = function(changedOptionIndex, callback) {
      //to trigger on change of catalog options
      product.changedOptionIndex = changedOptionIndex;
      var selected_options = [];
      _(product.co_ordering).forEach(function(obj) {
        selected_options.push(obj.selected_option.value);
      });
      if( !product.searchOption( selected_options ) ){
        selected_options = product.fallbackOption(selected_options);
      }
      product.setPreVisit(selected_options);
      var id = product.searchOption( selected_options );
      product.refresh_co_ordering(selected_options);
      product.set_current_variant(id);
      if(callback){
        callback();
      }
    } //Methods for Catalog Options End

    product.init = function(variant_id_to_set) {
      //decide first variant
      var init_variant = !variant_id_to_set ? _.first(product.variants) : _.find(product.variants, { 'sku' : variant_id_to_set });
      //initialize variant mapping
      if(!init_variant) return { 'error' : "Something went wrong! Please try again later." };
      product.init_mapping();
      product.cleanups();
      //sort
      _(product.co_ordering).forEach(function(co){
        co.options = _.sortBy(co.options, function(opt){
          //sort based on number if any any numering value is present inside the string
          var num_match = /((?:[a-zA-Z]|\s)*)(\d+)((?:\s|.)*)/.exec(opt.value);
          if(num_match)
            return parseFloat(num_match[2])
          else
            return opt.value
        })
      });
      //store co indexes for later use in pre-visits
      product.pre_visits = [];
      _(product.co_ordering).forEach(function(e,i){ product.pre_visits[i] = {}; e.index = i });
      //changedOption method scoped inside co ordering for use inside co ordering scope
      product.co_ordering.changedOption = product.changedOption;
      //Set Initial variant
      var selected_options = [];
      _(init_variant.catalog_options).forEach(function(opt){ selected_options.push(opt.value) });
      product.refresh_co_ordering(selected_options);
      product.set_current_variant(init_variant.sku);
      return product;
    }
    
    return product;
  }
}])

.factory('productCache', [ '$cacheFactory', function($cacheFactory){
  return $cacheFactory('product', { capacity : 15 })
}])

.service('productService',['babRA', 'productDecorator', 'productCache', function (babRA, productDecorator, productCache){
  var productRA = babRA.withConfig(function(RestangularConfigurer) {
    RestangularConfigurer.addElementTransformer('products', false, function (product) {
      return productDecorator(product);
    });
  });

  this.show = function (catalog_id){
    var query = { 'withRecommendation' : 'true' };
    return productRA.one("products",catalog_id).withHttpConfig({ cache : productCache ,timeout: 8000}).get(query);
  }
}])

.service('productReviewsService',['babRA', function (babRA){
  var reviewsRA = babRA.withConfig(function(RestangularConfigurer) {
    RestangularConfigurer.addElementTransformer('product-reviews', false, function (reviews) {
      return reviews;
    });
  });

  this.show = function (catalog_id){
    return reviewsRA.one("product-reviews",catalog_id).withHttpConfig({ timeout: 8000}).get();
  }
}])

.service('notifyMeService',['babRA', function (babRA){
  this.notifyTo = function (query){
    return babRA.one('tell-me-when-available').withHttpConfig({ timeout: 8000}).post(undefined, query);
  }
}])

.service('bulkProductService',['babRA', 'productDecorator', function (babRA, productDecorator){
  var productRA = babRA.withConfig(function(RestangularConfigurer) {
    RestangularConfigurer.addElementTransformer('random', false, function (products) {
      if(products.showBulk) {
        for (var i = 0; i < products.showBulk.length; i++) {
          products.showBulk[i] = productDecorator(products.showBulk[i]);
        };
      }
      return products;
    });
  });

  this.show = function (catalog_ids){
    return productRA.all("products").one("random", "showBulk").withHttpConfig({timeout: 8000}).get({"catalogIds" : catalog_ids});
  }
}])

.service('zipCheckService',['babRA', function (babRA){
  var productRA = babRA.withConfig(function(RestangularConfigurer) {
    RestangularConfigurer.addElementTransformer('checkzones', false, function (checkzones) {
      return checkzones;
    });
  });

  this.get = function (zipCode, listingId){
    return productRA.one("checkzones",zipCode).withHttpConfig().get({ 'listingId' : listingId });
  }
}])