angular.module('pnpSearch.directives', ['pnpCore'])
.directive('faSuspendable', function () {
  return {
    link: function (scope) {
      // Heads up: this might break is suspend/resume called out of order
      // or if watchers are added while suspended
      var watchers;

      scope.$on('suspend', function () {
        watchers = scope.$$watchers;
        scope.$$watchers = [];
      });

      scope.$on('resume', function () {
        if (watchers)
          scope.$$watchers = watchers;

        // discard our copy of the watchers
        watchers = void 0;
      });
    }
  };
})
.directive('ionAffix', ['$ionicPosition', function ($ionicPosition) {

        // keeping the Ionic specific stuff separated so that they can be changed and used within an other context

        // see https://api.jquery.com/closest/ and http://ionicframework.com/docs/api/utility/ionic.DomUtil/
        function getParentWithClass(elementSelector, parentClass) {
          return angular.element(ionic.DomUtil.getParentWithClass(elementSelector[0], parentClass));
        }

        // see http://underscorejs.org/#throttle
        function throttle(theFunction) {
          return ionic.Utils.throttle(theFunction);
        }

        // see http://www.paulirish.com/2011/requestanimationframe-for-smart-animating/
        // see http://ionicframework.com/docs/api/utility/ionic.DomUtil/
        function requestAnimationFrame(callback) {
          return ionic.requestAnimationFrame(callback);
        }

        // see https://api.jquery.com/offset/
        // see http://ionicframework.com/docs/api/service/$ionicPosition/
        function offset(elementSelector) {
          return $ionicPosition.offset(elementSelector);
        }

        // see https://api.jquery.com/position/
        // see http://ionicframework.com/docs/api/service/$ionicPosition/
        function position(elementSelector){
          return $ionicPosition.position(elementSelector);
        }

        function applyTransform(element, transformString) {
            // do not apply the transformation if it is already applied
            if (element.style[ionic.CSS.TRANSFORM] == transformString) {
            }
            else {
              element.style[ionic.CSS.TRANSFORM] = transformString;
            }
          }

          function translateUp(element, dy, executeImmediately) {
            var translateDyPixelsUp = dy == 0 ? 'translate3d(0px, 0px, 0px)' : 'translate3d(0px, -' + dy + 'px, 0px)';
            // if immediate execution is requested, then just execute immediately
            // if not, execute in the animation frame.
            if (executeImmediately) {
              applyTransform(element, translateDyPixelsUp);
            }
            else {
              // see http://www.paulirish.com/2011/requestanimationframe-for-smart-animating/
              // see http://ionicframework.com/docs/api/utility/ionic.DomUtil/
              requestAnimationFrame(function () {
                applyTransform(element, translateDyPixelsUp);
              });
            }
          }

          var CALCULATION_THROTTLE_MS = 500;

          return {
              // only allow adding this directive to elements as an attribute
              restrict: 'A',
              // we need $ionicScroll for adding the clone of affix element to the scroll container
              // $ionicScroll.element gives us that
              require: '^$ionicScroll',
              link: function ($scope, $element, $attr, $ionicScroll) {
                // get the affix's container. element will be affix for that container.
                // affix's container will be matched by "affix-within-parent-with-class" attribute.
                // if it is not provided, parent element will be assumed as the container
                var $container;
                if ($attr.affixWithinParentWithClass) {
                  $container = getParentWithClass($element, $attr.affixWithinParentWithClass);
                  if (!$container) {
                    $container = $element.parent();
                  }
                }
                else {
                  $container = $element.parent();
                }

                var scrollMin = 0;
                var scrollMax = 0;
                var scrollTransition = 0;
                // calculate the scroll limits for the affix element and the affix's container
                var calculateScrollLimits = function (scrollTop) {
                  var containerPosition = position($container);
                  var elementOffset = offset($element);
                  var containerTop = containerPosition.top;
                  var containerHeight = containerPosition.height;
                  var affixHeight = elementOffset.height;
                  scrollMin = scrollTop + containerTop;
                  scrollMax = scrollMin + containerHeight;
                  scrollTransition = scrollMax - affixHeight;
                };
              // throttled version of the same calculation
              var throttledCalculateScrollLimits = throttle(
                calculateScrollLimits,
                CALCULATION_THROTTLE_MS,
                {trailing: false}
                );
              var affixClone = null;
            // creates the affix clone and adds it to DOM. by default it is put to top
            var createAffixClone = function () {
              var clone = $element.clone(true).css({
                position: 'absolute',
                top: 0,
                left: 0,
                right: 0
              });
          // if directive is given an additional CSS class to apply to the clone, then apply it
          if ($attr.affixClass) {
            clone.addClass($attr.affixClass);
          }

          angular.element($ionicScroll.element).append(clone);

          return clone;
        };

        // removes the affix clone from DOM. also deletes the reference to it in the memory.
        var removeAffixClone = function () {
          if (affixClone)

            affixClone.remove();
          affixClone = null;
        };

        $scope.$on('$destroy', removeAffixClone);


        angular.element($ionicScroll.element).on('scroll', function (event) {
          var scrollTop = (event.detail || event.originalEvent && event.originalEvent.detail).scrollTop;
        // when scroll to top, we should always execute the immediate calculation.
        // this is because of some weird problem which is hard to describe.
        // if you want to experiment, always use the throttled one and just click on the page
        // you will see all affix elements stacked on top
        if (scrollTop == 0) {
          calculateScrollLimits(scrollTop);
        }
        else {
          throttledCalculateScrollLimits(scrollTop);
        }
        // when we scrolled to the container, create the clone of element and place it on top
        if (scrollTop >= scrollMin && scrollTop <= scrollMax) {

          // we need to track if we created the clone just now
          // that is important since normally we apply the transforms in the animation frame
          // but, we need to apply the transform immediately when we add the element for the first time. otherwise it is too late!
          var cloneCreatedJustNow = false;
          if (!affixClone) {
            affixClone = createAffixClone();
            cloneCreatedJustNow = true;
          }

          // if we're reaching towards the end of the container, apply some nice translation to move up/down the clone
          // but if we're reached already to the container and we're far away than the end, move clone to top
          if (scrollTop > scrollTransition) {
            translateUp(affixClone[0], Math.floor(scrollTop - scrollTransition), cloneCreatedJustNow);
          } else {
            translateUp(affixClone[0], 0, cloneCreatedJustNow);
          }
        } else {
          removeAffixClone();
        }
      });
}
}
}])


.directive('listItem', [function () {
  var listcontent={};
  listcontent.restrict = 'E';
  listcontent.templateUrl = "lib/pnp-search/templates/view_templates/list_template.html"
  return listcontent;
}])


.directive('gridItem', [function () {
  var gridcontent={};
  gridcontent.restrict = 'E';
  gridcontent.templateUrl = "lib/pnp-search/templates/view_templates/grid_template.html"
  return gridcontent;
}])



.directive('topCat',['browseNodeService',function(browseNodeService){
  var top_cat={};
  top_cat.restrict = 'E';
  top_cat.controller = function($scope){
    $scope.get_top_categories = function(){
      return browseNodeService.top_categories();
    }
  }
  top_cat.templateUrl = "lib/pnp-search/templates/top-cat.html";
  return top_cat;
}])

.directive('facetList',['browseNodeService',function(browseNodeService){
  var facet={};
  facet.restrict = 'E';
  facet.transclude = true;
  facet.controller = function($scope,$element,$ionicScrollDelegate){
    $element.on('scroll', function (e) {
        $scope.visibleList = getVisibleElements(e);
        $scope.$broadcast('suspend');
        $scope.$digest();
        $scope.$broadcast('resume');
    });
    
    $scope.facet_title = function(){
      if(browseNodeService.get_facet_current_bn()){
        return browseNodeService.get_facet_current_bn().display_name;
      }
      else{
        return "";
      }
    }

    $scope.get_browse_nodes = function(){
        return browseNodeService.browse_nodes();
    }

    $scope.addSingleSelectOption = function(facet_name,facet_value){
      $scope.$parent.$parent[facet_name.replace('prop.','')] = facet_value;
      browseNodeService.toggle_single_select(facet_name,facet_value);
    }

    $scope.checkSelected=function(facet_name,facet_value){
      if(browseNodeService.selected_filters[facet_name]!=null){
        if(browseNodeService.selected_filters[facet_name].indexOf(facet_value)>-1){
          return true;
        }
        else{
          return false;
        }
      }
      else{
        return false;
      }
    }
    
    $scope.selectionCount = function(key){
      selections=browseNodeService.selection();
      if(selections[key]!= null){
        return selections[key].length;
      }
      else{
        return 0;
      }
    }

    $scope.addOptions = function(facet_name,facet_value){
      browseNodeService.toggle(facet_name,facet_value);
    }

    $scope.toggleFacet = function(facet) {
      var current_facets = {};
      angular.copy(browseNodeService.search_filters(),current_facets);

      $ionicScrollDelegate.resize();
      if ($scope.isFacetShown(facet)) {
        if(!_.isEqual($scope.get_initial_facets(),current_facets)){
          browseNodeService.search_page(-1).then(function(){
            $scope.set_facet(browseNodeService.get_facet_list());
            $scope.set_initial_facets(browseNodeService.search_filters());
          })
        }
        $scope.shownFacet = null;
      } else {
        if($scope.shownFacet!=null && !_.isEqual($scope.get_initial_facets(),current_facets)){
          browseNodeService.search_page(-1).then(function(){
            $scope.set_facet(browseNodeService.get_facet_list());
            $scope.set_initial_facets(browseNodeService.search_filters());
          })
        }
        $scope.shownFacet = facet;
      }
    };

    $scope.isFacetShown = function(facet) {
      return $scope.shownFacet === facet;
    };

    $scope.get_selected_facet_string = function(facet){
      if(facet){
        return $scope.stringify_array(browseNodeService.selected_facet_vals(facet)) || "";
      }
    }
  };

  facet.templateUrl = "lib/pnp-search/templates/facet-list.html";
  return facet;
}])